# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'PostOwner', fields ['post', 'owner_content_type', 'owner_id']
        db.delete_unique('facebook_pages_postowner', ['post_id', 'owner_content_type_id', 'owner_id'])

        # Deleting model 'Post'
        db.delete_table('facebook_pages_post')

        # Deleting model 'PostOwner'
        db.delete_table('facebook_pages_postowner')

    def backwards(self, orm):
        # Adding model 'Post'
        db.create_table('facebook_pages_post', (
            ('picture', self.gf('django.db.models.fields.TextField')()),
            ('graph_id', self.gf('django.db.models.fields.CharField')(max_length=100, unique=True)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=500)),
            ('object_id', self.gf('django.db.models.fields.BigIntegerField')(null=True)),
            ('actions', self.gf('annoying.fields.JSONField')(null=True)),
            ('link', self.gf('django.db.models.fields.URLField')(max_length=500)),
            ('likes', self.gf('annoying.fields.JSONField')(null=True)),
            ('owners_json', self.gf('annoying.fields.JSONField')(null=True)),
            ('created_time', self.gf('django.db.models.fields.DateTimeField')()),
            ('author_json', self.gf('annoying.fields.JSONField')(null=True)),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('updated_time', self.gf('django.db.models.fields.DateTimeField')()),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('icon', self.gf('django.db.models.fields.URLField')(max_length=500)),
            ('story', self.gf('django.db.models.fields.CharField')(max_length=500)),
            ('privacy', self.gf('annoying.fields.JSONField')(null=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('caption', self.gf('django.db.models.fields.CharField')(max_length=500)),
            ('with_tags', self.gf('annoying.fields.JSONField')(null=True)),
            ('message_tags', self.gf('annoying.fields.JSONField')(null=True)),
            ('author_content_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='facebook_posts', null=True, to=orm['contenttypes.ContentType'])),
            ('comments', self.gf('annoying.fields.JSONField')(null=True)),
            ('properties', self.gf('annoying.fields.JSONField')(null=True)),
            ('application', self.gf('django.db.models.fields.related.ForeignKey')(related_name='posts', null=True, to=orm['facebook_applications.Application'])),
            ('comments_count', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('source', self.gf('django.db.models.fields.URLField')(max_length=500)),
            ('story_tags', self.gf('annoying.fields.JSONField')(null=True)),
            ('place', self.gf('annoying.fields.JSONField')(null=True)),
            ('author_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True)),
            ('likes_count', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('status_type', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal('facebook_pages', ['Post'])

        # Adding model 'PostOwner'
        db.create_table('facebook_pages_postowner', (
            ('owner_content_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='facebook_page_posts', null=True, to=orm['contenttypes.ContentType'])),
            ('post', self.gf('django.db.models.fields.related.ForeignKey')(related_name='owners', to=orm['facebook_pages.Post'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('owner_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True)),
        ))
        db.send_create_signal('facebook_pages', ['PostOwner'])

        # Adding unique constraint on 'PostOwner', fields ['post', 'owner_content_type', 'owner_id']
        db.create_unique('facebook_pages_postowner', ['post_id', 'owner_content_type_id', 'owner_id'])

    models = {
        'facebook_pages.page': {
            'Meta': {'ordering': "['name']", 'object_name': 'Page'},
            'about': ('django.db.models.fields.TextField', [], {}),
            'can_post': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'category': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'checkins': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'company_overview': ('django.db.models.fields.TextField', [], {}),
            'cover': ('annoying.fields.JSONField', [], {'null': 'True'}),
            'graph_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'likes': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '100'}),
            'location': ('annoying.fields.JSONField', [], {'null': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'picture': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'talking_about_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'website': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['facebook_pages']


INSTALLED_APPS = (
    'oauth_tokens',
    'taggit',
    'facebook_api',
    'facebook_posts',
    'facebook_users',
    'facebook_applications',
)

OAUTH_TOKENS_FACEBOOK_CLIENT_ID = 439210362826994
OAUTH_TOKENS_FACEBOOK_CLIENT_SECRET = '02569c93d18e4bc6aa210e594af84652'
OAUTH_TOKENS_FACEBOOK_SCOPE = ['offline_access']
OAUTH_TOKENS_FACEBOOK_USERNAME = '+919665223715'
OAUTH_TOKENS_FACEBOOK_PASSWORD = 'jcej9EIAQrrptDBY'

USE_TZ = True


import os
import sys
import argparse
from django.conf import settings

'''
QuickDjangoTest module for testing in Travis CI https://travis-ci.org
Changes log:
 * 2014-10-24 updated for compatibility with Django 1.7
 * 2014-11-03 different databases support: sqlite3, mysql, postgres
'''

class QuickDjangoTest(object):
    """
    A quick way to run the Django test suite without a fully-configured project.

    Example usage:

        >>> QuickDjangoTest('app1', 'app2')

    Based on a script published by Lukasz Dziedzia at:
    http://stackoverflow.com/questions/3841725/how-to-launch-tests-for-django-reusable-app
    """
    DIRNAME = os.path.dirname(__file__)
    INSTALLED_APPS = (
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
    )

    def __init__(self, *args, **kwargs):
        self.apps = args

        # Get the version of the test suite
        self.version = self.get_test_version()

        # Call the appropriate one
        if self.version == '1.7':
            self._tests_1_7()
        elif self.version == '1.2':
            self._tests_1_2()
        else:
            self._tests_old()

    def get_test_version(self):
        """
        Figure out which version of Django's test suite we have to play with.
        """
        from django import VERSION
        if VERSION[0] == 1 and VERSION[1] >= 7:
            return '1.7'
        elif VERSION[0] == 1 and VERSION[1] >= 2:
            return '1.2'
        else:
            return

    def get_database(self):
        test_db = os.environ.get('DB', 'sqlite')
        if test_db == 'mysql':
            database = {
                'ENGINE': 'django.db.backends.mysql',
                'NAME': 'django',
                'USER': 'root',
            }
        elif test_db == 'postgres':
            database = {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'USER': 'postgres',
                'NAME': 'django',
                'OPTIONS': {
                    'autocommit': True,
                }
            }
        else:
            database = {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(self.DIRNAME, 'database.db'),
                'USER': '',
                'PASSWORD': '',
                'HOST': '',
                'PORT': '',
            }
        return {'default': database}

    def get_custom_settings(self):
        try:
            from settings_test import *
            settings_test = dict(locals())
            del settings_test['self']
            if 'INSTALLED_APPS' in settings_test:
                del settings_test['INSTALLED_APPS']
        except ImportError:
            settings_test = {}
            INSTALLED_APPS = []

        return INSTALLED_APPS, settings_test

    def _tests_old(self):
        """
        Fire up the Django test suite from before version 1.2
        """
        INSTALLED_APPS, settings_test = self.get_custom_settings()

        settings.configure(DEBUG = True,
            DATABASE_ENGINE = 'sqlite3',
            DATABASE_NAME = os.path.join(self.DIRNAME, 'database.db'),
            INSTALLED_APPS = self.INSTALLED_APPS + INSTALLED_APPS + self.apps,
            **settings_test
        )
        from django.test.simple import run_tests
        failures = run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def _tests_1_2(self):
        """
        Fire up the Django test suite developed for version 1.2 and up
        """
        INSTALLED_APPS, settings_test = self.get_custom_settings()

        settings.configure(
            DEBUG = True,
            DATABASES = self.get_database(),
            INSTALLED_APPS = self.INSTALLED_APPS + INSTALLED_APPS + self.apps,
            **settings_test
        )

        from django.test.simple import DjangoTestSuiteRunner
        failures = DjangoTestSuiteRunner().run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def _tests_1_7(self):
        """
        Fire up the Django test suite developed for version 1.7 and up
        """
        INSTALLED_APPS, settings_test = self.get_custom_settings()

        settings.configure(
            DEBUG = True,
            DATABASES = self.get_database(),
            MIDDLEWARE_CLASSES = ('django.middleware.common.CommonMiddleware',
                                  'django.middleware.csrf.CsrfViewMiddleware'),
            INSTALLED_APPS = self.INSTALLED_APPS + INSTALLED_APPS + self.apps,
            **settings_test
        )

        from django.test.simple import DjangoTestSuiteRunner
        import django
        django.setup()
        failures = DjangoTestSuiteRunner().run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)


if __name__ == '__main__':
    """
    What do when the user hits this file from the shell.

    Example usage:

        $ python quicktest.py app1 app2

    """
    parser = argparse.ArgumentParser(
        usage="[args]",
        description="Run Django tests on the provided applications."
    )
    parser.add_argument('apps', nargs='+', type=str)
    args = parser.parse_args()
    QuickDjangoTest(*args.apps)



# -*- coding: utf-8 -*-
from BeautifulSoup import BeautifulSoup
from django.test import TestCase
from facebook_api.utils import get_or_create_from_small_resource
from facebook_posts.models import Post

from .factories import PageFactory
from .models import Page, PAGES_FANS_USER_ID
from .parser import FacebookPageFansParser

PAGE_ID = '19292868552'
PAGE_RESOURCE_SHORT = {'category': 'Product/service', 'id': PAGE_ID, 'name': 'Facebook Developers'}
PAGE_URL = 'https://www.facebook.com/pages/METRO-Cash-and-Carry-Russia/129107667156177'
PAGE_FANS_ID = 501842786534856


class FacebookPagesTest(TestCase):

    def test_fetch_page(self):

        self.assertEqual(Page.objects.count(), 0)
        page = Page.remote.fetch(PAGE_ID)
        page = Page.remote.fetch(PAGE_ID)
        self.assertEqual(Page.objects.count(), 1)

        self.assertEqual(page.graph_id, PAGE_ID)
        self.assertEqual(page.name, 'Facebook Developers')
        self.assertEqual(page.is_published, True)
        self.assertEqual(page.website, 'http://developers.facebook.com')
        self.assertEqual(page.category, "Product/service")
        self.assertEqual(page.username, 'FacebookDevelopers')
        self.assertEqual(page.link, 'https://www.facebook.com/FacebookDevelopers')
        self.assertGreater(len(page.company_overview), 0)
        self.assertGreater(page.likes_count, 0)

        page.username = page.website = ''
        self.assertEqual(page.username, '')
        self.assertEqual(page.website, '')
        page.save()
        page = Page.remote.fetch(PAGE_ID)
        self.assertEqual(page.username, 'FacebookDevelopers')
        self.assertEqual(page.website, 'http://developers.facebook.com')

    def test_get_by_url(self):

        page = Page.remote.get_by_url('https://www.facebook.com/pages/METRO-Cash-and-Carry-Russia/129107667156177')

        self.assertEqual(page.graph_id, '129107667156177')
        self.assertEqual(page.name, 'METRO Cash and Carry Russia')
        self.assertEqual(page.is_published, True)
        self.assertEqual(page.website, 'http://www.metro-cc.ru')

    def test_fetch_page_from_resource(self):

        Page.remote.fetch(PAGE_ID)

        page = get_or_create_from_small_resource(PAGE_RESOURCE_SHORT)
        self.assertEqual(page.name, PAGE_RESOURCE_SHORT['name'])
        self.assertEqual(page.category, PAGE_RESOURCE_SHORT['category'])

        page1 = Page.objects.all()[0]
        self.assertEqual(page1.website, "http://developers.facebook.com")

        page2 = get_or_create_from_small_resource(PAGE_RESOURCE_SHORT)
        self.assertEqual(page2.website, "http://developers.facebook.com")
        self.assertEqual(page2.category, PAGE_RESOURCE_SHORT['category'])


class FacebookPageFansTest(TestCase):

    def test_get_parser_response(self):

        parser = FacebookPageFansParser(
            authorized=True, url='/ajax/browser/list/page_fans/?page_id=%s&start=0&__user=%s&__a=1' % (PAGE_FANS_ID, PAGES_FANS_USER_ID))
        self.assertIsInstance(parser.content_bs, BeautifulSoup)

    def test_fetch_fans_ids(self):

        page = PageFactory(graph_id=PAGE_FANS_ID)

        ids = page.fetch_fans_ids_parser()
        self.assertGreater(len(ids), 450)



# -*- coding: utf-8 -*-
import logging
import re

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from facebook_api import fields
from facebook_api.decorators import atomic
from facebook_api.models import FacebookGraphIDModel, FacebookGraphManager
from facebook_api.utils import graph

from .parser import FacebookPageFansParser, FacebookParseError


log = logging.getLogger('facebook_pages')

PAGES_FANS_USER_ID = getattr(settings, 'FACEBOOK_PAGES_FANS_USER_ID', '')


class FacebookPageGraphManager(FacebookGraphManager):

    def get_by_slug(self, slug):
        '''
        Return object by slug
        '''
        # no slug pages - https://www.facebook.com/pages/METRO-Cash-and-Carry-Russia/129107667156177
        if slug[:5] == 'pages':
            m = re.findall(r'^pages/.+/(\d+)', slug)
            slug = m[0]
        return self.fetch(slug)


class Page(FacebookGraphIDModel):

    name = models.CharField(max_length=200, help_text='The Page\'s name')
    link = models.URLField(max_length=1000, help_text='Link to the page on Facebook')

    is_published = models.BooleanField(
        help_text='Indicates whether the page is published and visible to non-admins', default=False)
    can_post = models.BooleanField(
        help_text='Indicates whether the current session user can post on this Page', default=False)

    location = fields.JSONField(
        null=True, help_text='The Page\'s street address, latitude, and longitude (when available)')
    cover = fields.JSONField(
        null=True, help_text='The JSON object including cover_id (the ID of the photo), source (the URL for the cover photo), and offset_y (the percentage offset from top [0-100])')

    likes_count = models.IntegerField(null=True, help_text='The number of users who like the Page')
    checkins_count = models.IntegerField(
        null=True, help_text='The total number of users who have checked in to the Page')
    talking_about_count = models.IntegerField(
        null=True, help_text='The number of people that are talking about this page (last seven days)')

    category = models.CharField(max_length=100, help_text='The Page\'s category')
    phone = models.CharField(
        max_length=100, help_text='The phone number (not always normalized for country code) for the Page')
    # If the "October 2012 Breaking Changes" migration setting is enabled for
    # your app, this field will be an object with the url and is_silhouette
    # fields; is_silhouette is true if the user has not uploaded a profile
    # picture
    picture = models.CharField(max_length=100, help_text='Link to the Page\'s profile picture')
    website = models.CharField(max_length=1000, help_text='Link to the external website for the page')

    # for managing pages
    #access_token = models.CharField(max_length=500, help_text='A Page admin access_token for this page; The current user must be an administrator of this page; only returned if specifically requested via the fields URL parameter')

    # not in API
    username = models.CharField(max_length=200)
    company_overview = models.TextField()
    about = models.TextField()
    products = models.TextField()
    description = models.TextField()

    # auto-estimated values
    posts_count = models.IntegerField(default=0)

    objects = models.Manager()
    remote = FacebookPageGraphManager()

    class Meta:
        verbose_name = 'Facebook page'
        verbose_name_plural = 'Facebook pages'

    def __unicode__(self):
        return self.name

    def parse(self, response):

        for field_name in ['likes', 'checkins']:
            response['%s_count' % field_name] = response.get(field_name)

        super(Page, self).parse(response)

    @property
    def slug(self):
        return self.username or self.graph_id

    @property
    def wall_posts(self):
        if 'facebook_posts' not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured("Application 'facebook_posts' not in INSTALLED_APPS")

        from facebook_posts.models import Post
        # .(owners__owner_content_type=ContentType.objects.get_for_model(Page), owners__owner_id=self.id)
        return Post.objects.filter(graph_id__startswith='%s_' % self.graph_id)

    @property
    def wall_comments(self):
        if 'facebook_posts' not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured("Application 'facebook_posts' not in INSTALLED_APPS")

        from facebook_posts.models import Comment
        return Comment.objects.filter(graph_id__startswith='%s_' % self.graph_id)

    @atomic
    def fetch_posts(self, *args, **kwargs):
        '''
        Retrieve and save all posts of page
        '''
        if 'facebook_posts' not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured("Application 'facebook_posts' not in INSTALLED_APPS")

        from facebook_posts.models import Post
        return Post.remote.fetch_page(page=self, *args, **kwargs)

#    @atomic
    def fetch_fans(self, *args, **kwargs):
        return self.fetch_fans_ids_parser()

    def fetch_fans_ids_parser(self):
        ids = []
        offset = 0
        parser = FacebookPageFansParser()
        while True:
            try:
                users = self.parse_fans(parser, offset)
                assert len(users) > 0
            except (FacebookParseError, AssertionError):
                break

            ids += [user.graph_id for user in users]
            offset += 10

#            print offset, len(ids)

        return ids

    def parse_fans(self, parser, offset=0):

        if 'facebook_users' not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured("Application 'facebook_users' not in INSTALLED_APPS")
        from facebook_users.models import User

        parser.request(authorized=True, url='/ajax/browser/list/page_fans/?page_id=%s&start=%s&__user=%s&__a=1' %
                       (self.graph_id, offset, PAGES_FANS_USER_ID))

        users = []
        for item in parser.content_bs.findAll('li', {'class': 'adminableItem fbProfileBrowserListItem'}):
            initial = {'graph_id': item['id'].replace('adminableItem_', '')}
            name = item.find('div', {'class': 'uiProfileBlockContent'}).find('a').text

            # https://www.facebook.com/milkamelot?fref=pb
            href = item.find('a')['href']
            if 'profile.php' in href:
                initial['username'] = href.split('?')[0].split('/')[3]

            user = User(**initial)
            user.set_name(name)
            users += [user]

        return users



# -*- coding: utf-8 -*-
from facebook_api.parser import FacebookParser, FacebookParseError
from django.utils import simplejson as json

class FacebookPageFansParser(FacebookParser):

    @property
    def html(self):
        content = self.content
        response_instance = json.loads(content[9:])
        try:
            content = response_instance['domops'][0][3]['__html']
        except KeyError:
            if 'error' in response_instance:
                raise FacebookParseError(response_instance['error'], response_instance['errorDescription'])
            else:
                raise

        return content